/*
 * Trivial control program to get information and turn the device off/onn
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/extensions/XI.h>
#include <X11/extensions/XInput.h>

char *prgname;

int gunze_getinfo(Display *disp)
{
    XExtensionVersion *ext;
    XDeviceInfo *info;
    int infolen, i;

    ext = XGetExtensionVersion(disp, INAME);
    printf("%s: %s", INAME, ext->present ? "Present" : "Absent");
    if (ext->present)
	printf(" (version %i.%i)", ext->major_version, ext->minor_version);
    printf("\n");
    XFree(ext);

    info = XListInputDevices(disp, &infolen);
    for (i=0; i<infolen; i++) {
	printf("Device %s (id %i): %i classes, use is %s\n",
	       info[i].name, (int)(info[i].id),
	       info[i].num_classes,
	       info[i].use == IsXKeyboard ? "keyboard" :
	       (info[i].use == IsXPointer ? "pointer" :
		(info[i].use == IsXExtensionDevice ? "extension" :
		 "unknown")));
    }
    XFreeDeviceList(info);
    return 0;
}

int gunze_finddevice(Display *disp)
{
    XDeviceInfo *info;
    int infolen, i;

    info = XListInputDevices(disp, &infolen);
    for (i=0; i<infolen; i++) {
	if (info[i].use != IsXExtensionDevice)
	    continue;
	if (strstr(info[i].name, "touch"))
	    break;
	if (strstr(info[i].name, "Touch"))
	    break;
    }
    if (i == infolen)
	i = -1 /* not found */;
    else
	i = info[i].id;
    /* XFreeDeviceList(info); */
    return i;
}


int gunze_deviceon(Display *disp, int on)
{
    XDevice *dev;
    int id = gunze_finddevice(disp);
    XDeviceResolutionControl ctl;
    int resols[2];

    if (id < 0) {
	fprintf(stderr, "%s: can't find a touch screen\n", prgname);
	return 1;
    }
    dev = XOpenDevice(disp, id);
    printf("device %i: %p\n", id, dev);

    ctl.control = DEVICE_RESOLUTION;
    ctl.length = sizeof(ctl);
    ctl.first_valuator = 0;
    ctl.num_valuators = 1 + on;
    ctl.resolutions = resols;
    resols[0]=resols[1]=1000; /* anything (in 0-500k) */

#if 0
    XSetDeviceMode(disp, dev, Absolute);
#else
    XChangeDeviceControl(disp, dev, DEVICE_RESOLUTION, (XDeviceControl *)&ctl);
#endif

    XCloseDisplay(disp);
    return 0;
}


int main (int argc, char **argv)
{
    Display *disp;

    prgname = argv[0];

    if (argc != 2) {
	fprintf(stderr, "%s: Use \"%s <cmd>\" (\"info\", \"off\", \"on\")\n",
		prgname, prgname);
	exit(1);
    }

    /* Open display, once for all */
    disp = XOpenDisplay(NULL);
    if (!disp) {
	fprintf(stderr, "%s: can't open display\n", prgname);
	exit(1);
    }

    if (!strcmp(argv[1], "info"))
	return gunze_getinfo(disp);
    if (!strcmp(argv[1], "off"))
	return gunze_deviceon(disp, 0);
    if (!strcmp(argv[1], "on"))
	return gunze_deviceon(disp, 1);
    
    fprintf(stderr, "%s: Use \"%s <cmd>\" (\"info\", \"off\", \"on\")\n",
	    prgname, prgname);
    exit(1);
}
